@@ -1,4 +1,4 @@
-# Copyright (C) 2005 Warp Networks S.L., DBS Servicios Informaticos S.L.
+# Copyright (C) 2008-2010 eBox Technologies S.L.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License, version 2, as
@@ -37,6 +37,7 @@ use EBox::Firewall::Model::InternalToEBoxRuleTable;
 use EBox::Firewall::Model::ExternalToEBoxRuleTable;
 use EBox::Firewall::Model::EBoxOutputRuleTable;
 use EBox::Firewall::Model::ExternalToInternalRuleTable;
+use EBox::Firewall::Model::EBoxServicesRuleTable;
 use EBox::Firewall::Model::RedirectsTable;
 
 use EBox::Firewall::Model::Report::PacketTrafficDetails;
@@ -56,59 +57,17 @@ sub _create
                                      domain => 'ebox-firewall',
                                      printableName => __n('Firewall'),
                                      @_);
-    $self->{'ToInternetRuleModel'} =
-        new EBox::Firewall::Model::ToInternetRuleTable(
-                                                       'gconfmodule' => $self,
-                                                       'directory' => 'ToInternetRuleTable',
-                                                      );
-
-    $self->{'InternalToEBoxRuleModel'} =
-        new EBox::Firewall::Model::InternalToEBoxRuleTable(
-                                                           'gconfmodule' => $self,
-                                                           'directory' => 'InternalToEBoxRuleTable',
-                                                          );
-
-    $self->{'ExternalToEBoxRuleModel'} =
-        new EBox::Firewall::Model::ExternalToEBoxRuleTable(
-                                                           'gconfmodule' => $self,
-                                                           'directory' => 'ExternalToEBoxRuleTable',
-                                                          );
-
-    $self->{'EBoxOutputRuleModel'} =
-        new EBox::Firewall::Model::EBoxOutputRuleTable(
-                                                       'gconfmodule' => $self,
-                                                       'directory' => 'EBoxOutputRuleTable',
-                                                      );
-    $self->{'ExternalToInternalRuleTable'} =
-        new EBox::Firewall::Model::ExternalToInternalRuleTable(
-                                                               'gconfmodule' => $self,
-                                                               'directory' => 'ExternalToInternalRuleTable',
-                                                              );
-    $self->{'RedirectsTable'} =
-        new EBox::Firewall::Model::RedirectsTable(
-                                                     'gconfmodule' => $self,
-                                                     'directory' => 'RedirectsTable',
-                                                    );
-
-
-    $self->{'PacketTrafficDetails'} = new EBox::Firewall::Model::Report::PacketTrafficDetails (
-                                                                                               'gconfmodule' => $self,
-                                                                                               'directory' => 'PacketTrafficDetails',
-                                                                                              );
-
-    $self->{'PacketTrafficGraph'} = new EBox::Firewall::Model::Report::PacketTrafficGraph (
-                                                                                           'gconfmodule' => $self,
-                                                                                           'directory' => 'PacketTrafficGraph',
-                                                                                          );
-
-    $self->{'PacketTrafficReportOptions'} = new EBox::Firewall::Model::Report::PacketTrafficReportOptions (
-                                                                                                           'gconfmodule' => $self,
-                                                                                                           'directory' => 'PacketTrafficReportOptions',
-                                                                                                          );
-
-
-
 
+    $self->{'ToInternetRuleModel'} = $self->model('ToInternetRuleTable');
+    $self->{'InternalToEBoxRuleModel'} = $self->model('InternalToEBoxRuleTable');
+    $self->{'ExternalToEBoxRuleModel'} = $self->model('ExternalToEBoxRuleTable');
+    $self->{'EBoxOutputRuleModel'} = $self->model('EBoxOutputRuleTable');
+    $self->{'ExternalToInternalRuleTable'} = $self->model('ExternalToInternalRuleTable');
+    $self->{'EBoxServicesRuleTable'} = $self->model('EBoxServicesRuleTable');
+    $self->{'RedirectsTable'} = $self->model('RedirectsTable');
+    $self->{'PacketTrafficDetails'} = $self->model('PacketTrafficDetails');
+    $self->{'PacketTrafficGraph'} = $self->model('PacketTrafficGraph');
+    $self->{'PacketTrafficReportOptions'} = $self->model('PacketTrafficReportOptions');
 
     bless($self, $class);
     return $self;
@@ -123,7 +82,7 @@ sub actions
     return [
         {
                 'action' => __('Flush previous firewall rules'),
-                'reason' => __('The eBox firewall will flush any previous firewall '
+                'reason' => __('The Zentyal firewall will flush any previous firewall '
                                . 'rules which have been added manually or by another tool'),
                 'module' => 'firewall'
         },
@@ -140,31 +99,29 @@ sub actions
         ];
 }
 
-# Method: models
+# Method: modelClasses
 #
-#      Overrides <EBox::Model::ModelProvider::models>
+#      Overrides <EBox::Model::ModelProvider::modelClasses>
 #
-sub models
+sub modelClasses
 {
     my ($self) = @_;
 
     return [
-            $self->{'ToInternetRuleModel'},
-            $self->{'InternalToEBoxRuleModel'},
-            $self->{'ExternalToEBoxRuleModel'},
-            $self->{'EBoxOutputRuleModel'},
-            $self->{'ExternalToInternalRuleTable'},
-            $self->{'RedirectsTable'},
-
-            $self->{'PacketTrafficDetails'},
-            $self->{'PacketTrafficGraph'},
-            $self->{'PacketTrafficReportOptions'},
+            'EBox::Firewall::Model::ToInternetRuleTable',
+            'EBox::Firewall::Model::InternalToEBoxRuleTable',
+            'EBox::Firewall::Model::ExternalToEBoxRuleTable',
+            'EBox::Firewall::Model::EBoxOutputRuleTable',
+            'EBox::Firewall::Model::ExternalToInternalRuleTable',
+            'EBox::Firewall::Model::EBoxServicesRuleTable',
+            'EBox::Firewall::Model::RedirectsTable',
+            'EBox::Firewall::Model::Report::PacketTrafficDetails',
+            'EBox::Firewall::Model::Report::PacketTrafficGraph',
+            'EBox::Firewall::Model::Report::PacketTrafficReportOptions'
            ];
 }
 
 
-
-
 sub compositeClasses
 {
     return [
@@ -646,13 +603,13 @@ sub addOutputRule # (protocol, port)
 
 # Method: setInternalService
 #
-#   This method adds a rule to the "internal networks to eBox services"
+#   This method adds a rule to the "internal networks to Zentyal services"
 #   table.
 #
 #   In case the service has already been configured with a custom
 #   rule by the user the adding operation is aborted.
 #
-#   Modules configuring internal services running on eBox should use
+#   Modules configuring internal services running on Zentyal should use
 #   this method if they wish to allow access from internal networks
 #   to the service by default.
 #
@@ -679,13 +636,13 @@ sub setInternalService
 
 # Method: setExternalService
 #
-#   This method adds a rule to the "external networks to eBox services"
+#   This method adds a rule to the "external networks to Zentyal services"
 #   table.
 #
 #   In case the service has already been configured with a custom
 #   rule by the user the adding operation is aborted.
 #
-#   Modules configuring internal services running on eBox should use
+#   Modules configuring internal services running on Zentyal should use
 #   this method if they wish to allow access from external networks
 #   to the service by default.
 #
@@ -777,7 +734,7 @@ sub enableLog
 #   This method is used to enable/disable the iptables logging facilities.
 #
 #   When enabled, it will log drop packets to syslog, and they will be
-#   introduced into the eBox log DB.
+#   introduced into the Zentyal log DB.
 #
 # Parameters:
 #
@@ -849,7 +806,7 @@ sub tableInfo
 
     my @order = qw(timestamp fw_in fw_out fw_src fw_dst fw_proto fw_spt fw_dpt event);
 
-    my $events = { 'drop' => __('DROP'), 'log' => __('LOG') };
+    my $events = { 'drop' => __('DROP'), 'log' => __('LOG'), 'redirect' => __('REDIRECT'), };
 
     return [{
             'name' => __('Firewall'),
