@@ -1,4 +1,4 @@
-# Copyright (C) 2007 Warp Networks S.L.
+# Copyright (C) 2008-2010 eBox Technologies S.L.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License, version 2, as
@@ -143,7 +143,13 @@ sub setService
                                              . $iptables);
             }
 
-        } elsif ($protocol eq any ('gre', 'icmp', 'esp')) {
+        } elsif ($protocol eq 'icmp') {
+            my @icmp_types = qw(echo-request echo-reply destination-unreachable source-quench parameter-problem);
+            foreach my $type (@icmp_types) {
+                $iptables = " -p $invProto $protocol --icmp-type $type ! -f";
+                push (@{$self->{'service'}}, $iptables);
+            }
+        } elsif ($protocol eq any ('gre', 'esp')) {
             $iptables = " -p $invProto $protocol";
             push (@{$self->{'service'}}, $iptables);
         } elsif ($protocol eq 'any') {
