@@ -1,4 +1,4 @@
-# Copyright (C) 2005 Warp Networks S.L., DBS Servicios Informaticos S.L.
+# Copyright (C) 2008-2010 eBox Technologies S.L.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License, version 2, as
@@ -183,7 +183,7 @@ sub _fieldDescription
             new EBox::Types::Text(
                 'fieldName' => 'description',
                 'printableName' => __('Description'),
-                'size' => '15',
+                'size' => '32',
                 'editable' => 1,
                 'optional' => 1,
                 ),
@@ -227,4 +227,24 @@ sub headTitle
     return __d('Configure Rules', 'ebox-firewall');
 }
 
+
+sub validateTypedRow
+{
+    my ($self, $action, $params_r, $actual_r) = @_;
+
+    if ($params_r->{service}) {
+        my $service = $params_r->{service};
+        # don't allow inverse match of any service
+
+        if ($service->inverseMatch()) {
+            if ($service->printableValue eq '! any') {
+                throw EBox::Exceptions::External(
+__(q{'any service' cannot have a inverse march})
+                                                );
+            }
+        }
+    }
+}
+
+
 1;
