@@ -1,4 +1,4 @@
-# Copyright (C) 2005 Warp Networks S.L., DBS Servicios Informaticos S.L.
+# Copyright (C) 2008-2010 eBox Technologies S.L.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License, version 2, as
@@ -15,7 +15,7 @@
 
 # Class: EBox::Firewall::Model::EBoxOutputRuleTable
 #
-# This class describes the model used to store rules to allow eBox to
+# This class describes the model used to store rules to allow Zentyal to
 # access external services
 #
 # Inherits from <EBox::Firewall::Model::BaseRuleTable> to fetch
@@ -65,7 +65,7 @@ sub _table
     {
         'tableName' => 'EBoxOutputRuleTable',
         'printableTableName' =>
-          __('Traffic coming out from eBox'),
+          __('Traffic coming out from Zentyal'),
         'automaticRemove' => 1,
         'defaultController' =>
             '/ebox/Firewall/Controller/EBoxOutputRuleTable',
