@@ -1,4 +1,4 @@
-# Copyright (C) 2005 Warp Networks S.L., DBS Servicios Informaticos S.L.
+# Copyright (C) 2008-2010 eBox Technologies S.L.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License, version 2, as
@@ -16,7 +16,7 @@
 # Class: EBox::Firewall::Model::ExternalToEBoxRuleTable
 #
 # This class describes the model used to store rules to access
-# services running on eBox from internal networks.
+# services running on Zentyal from internal networks.
 #
 # Inherits from <EBox::Firewall::Model::BaseRuleTable> to fetch
 # the field description
@@ -65,7 +65,7 @@ sub _table
     {
         'tableName' => 'ExternalToEBoxRuleTable',
         'printableTableName' =>
-          __('External networks to eBox'),
+          __('External networks to Zentyal'),
         'automaticRemove' => 1,
         'defaultController' =>
             '/ebox/Firewall/Controller/ExternalToEBoxRuleTable',
