@@ -1,4 +1,4 @@
-# Copyright (C) 2005 Warp Networks S.L., DBS Servicios Informaticos S.L.
+# Copyright (C) 2008-2010 eBox Technologies S.L.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License, version 2, as
@@ -258,7 +258,7 @@ sub _fieldDescription
             [
             new EBox::Types::Union::Text(
                 'fieldName' => 'origDest_ebox',
-                'printableName' => __('eBox')),
+                'printableName' => 'Zentyal'),
             new EBox::Types::IPAddr(
                 'fieldName' => 'origDest_ipaddr',
                 'printableName' => __('IP Address'),
@@ -332,10 +332,17 @@ sub _fieldDescription
             'editable' => 1);
     push (@tableHead, $dport);
 
+    my $plog = new EBox::Types::Boolean(
+            'fieldName' => 'log',
+            'printableName' => __('Log'),
+            'editable' => 1,
+            'help' => __('Log new forwarded connections'));
+    push (@tableHead, $plog);
+
     my $desc = new EBox::Types::Text(
             'fieldName' => 'description',
             'printableName' => __('Description'),
-            'size' => '15',
+            'size' => '32',
             'editable' => 1,
             'optional' => 1);
     push (@tableHead, $desc);
