@@ -1,4 +1,4 @@
-# Copyright (C) 2005 Warp Networks S.L., DBS Servicios Informaticos S.L.
+# Copyright (C) 2008-2010 eBox Technologies S.L.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License, version 2, as
@@ -36,7 +36,7 @@ use Error qw( :try );
 use Perl6::Junction qw( any );
 use EBox::Sudo qw( :all );
 
-my $new = " -m state --state NEW ";
+my $statenew = " -m state --state NEW ";
 
 use constant IPT_MODULES => ('ip_conntrack_ftp', 'ip_nat_ftp', 'ip_conntrack_tftp');
 use constant SYSLOG_LEVEL => 7;
@@ -158,14 +158,13 @@ sub _setStructure
 
     # state rules
     push(@commands,
+            pf('-A OUTPUT -m state --state INVALID -j DROP'),
             pf('-A OUTPUT -m state --state ESTABLISHED,RELATED -j ACCEPT'),
+            pf('-A INPUT -m state --state INVALID -j DROP'),
             pf('-A INPUT -m state --state ESTABLISHED,RELATED -j ACCEPT'),
+            pf('-A FORWARD -m state --state INVALID -j DROP'),
             pf('-A FORWARD -m state --state ESTABLISHED,RELATED -j ACCEPT'),
 
-            pf('-A OUTPUT -p icmp ! -f -j ACCEPT'),
-            pf('-A INPUT -p icmp ! -f -j ACCEPT'),
-            pf('-A FORWARD -p icmp ! -f -j ACCEPT'),
-
             pf('-t nat -N premodules'),
 
             pf('-t nat -N postmodules'),
@@ -212,6 +211,12 @@ sub _setStructure
             pf('-A FORWARD -j fdns'),
             pf('-A FORWARD -j fobjects'),
             pf('-A FORWARD -j fglobal'),
+            pf("-A FORWARD -p icmp --icmp-type echo-request ! -f $statenew -j ACCEPT"), # accept ping requests
+            pf("-A FORWARD -p icmp --icmp-type echo-reply ! -f $statenew -j ACCEPT"), # accept ping responses
+            pf("-A FORWARD -p icmp --icmp-type destination-unreachable ! -f $statenew -j ACCEPT"), # accept notifications of unreachable hosts
+            pf("-A FORWARD -p icmp --icmp-type source-quench ! -f $statenew -j ACCEPT"), # accept notifications to reduce sending speed
+            pf("-A FORWARD -p icmp --icmp-type time-exceeded ! -f $statenew -j ACCEPT"), # accept notifications of lost packets
+            pf("-A FORWARD -p icmp --icmp-type parameter-problem ! -f $statenew -j ACCEPT"), # accept notifications of protocol problems
             pf('-A FORWARD -j fdrop'),
 
             pf('-A INPUT -j inospoof'),
@@ -221,11 +226,23 @@ sub _setStructure
             pf('-A INPUT -j imodules'),
             pf('-A INPUT -j iintservs'),
             pf('-A INPUT -j iglobal'),
+            pf("-A INPUT -p icmp --icmp-type echo-request ! -f $statenew -j ACCEPT"), # accept ping requests
+            pf("-A INPUT -p icmp --icmp-type echo-reply ! -f $statenew -j ACCEPT"), # accept ping responses
+            pf("-A INPUT -p icmp --icmp-type destination-unreachable ! -f $statenew -j ACCEPT"), # accept notifications of unreachable hosts
+            pf("-A INPUT -p icmp --icmp-type source-quench ! -f $statenew -j ACCEPT"), # accept notifications to reduce sending speed
+            pf("-A INPUT -p icmp --icmp-type time-exceeded ! -f $statenew -j ACCEPT"), # accept notifications of lost packets
+            pf("-A INPUT -p icmp --icmp-type parameter-problem ! -f $statenew -j ACCEPT"), # accept notifications of protocol problems
             pf('-A INPUT -j idrop'),
 
             pf('-A OUTPUT -j ointernal'),
             pf('-A OUTPUT -j omodules'),
             pf('-A OUTPUT -j oglobal'),
+            pf("-A OUTPUT -p icmp --icmp-type echo-request ! -f $statenew -j ACCEPT"), # accept ping requests
+            pf("-A OUTPUT -p icmp --icmp-type echo-reply ! -f $statenew -j ACCEPT"), # accept ping responses
+            pf("-A OUTPUT -p icmp --icmp-type destination-unreachable ! -f $statenew -j ACCEPT"), # accept notifications of unreachable hosts
+            pf("-A OUTPUT -p icmp --icmp-type source-quench ! -f $statenew -j ACCEPT"), # accept notifications to reduce sending speed
+            pf("-A OUTPUT -p icmp --icmp-type time-exceeded ! -f $statenew -j ACCEPT"), # accept notifications of lost packets
+            pf("-A OUTPUT -p icmp --icmp-type parameter-problem ! -f $statenew -j ACCEPT"), # accept notifications of protocol problems
             pf('-A OUTPUT -j odrop'),
 
             pf("-A idrop -j drop"),
@@ -248,8 +265,10 @@ sub _setDNS # (dns)
     my ($self, $dns) = @_;
 
     my @commands = (
-            pf("-A ointernal $new -p udp --dport 53 -d $dns -j ACCEPT"),
-            pf("-A fdns $new -p udp --dport 53 -d $dns -j ACCEPT"),
+            pf("-A ointernal $statenew -p udp --dport 53 -d $dns -j ACCEPT"),
+            pf("-A ointernal $statenew -p tcp --dport 53 -d $dns -j ACCEPT"),
+            pf("-A fdns $statenew -p udp --dport 53 -d $dns -j ACCEPT"),
+            pf("-A fdns $statenew -p tcp --dport 53 -d $dns -j ACCEPT"),
             );
     return \@commands;
 }
@@ -267,7 +286,7 @@ sub _setDHCP
     my ($self, $interface) = @_;
 
     $interface = $self->{net}->realIface($interface);
-    return [ pf("-A ointernal $new -o $interface -p udp --dport 67 -j ACCEPT") ];
+    return [ pf("-A ointernal $statenew -o $interface -p udp --dport 67 -j ACCEPT") ];
 }
 
 # Method: _setRemoteServices
@@ -287,18 +306,18 @@ sub _setRemoteServices
         if ( $rsMod->eBoxSubscribed() ) {
             my $vpnIface = $rsMod->ifaceVPN();
             push(@commands,
-                pf("-A ointernal $new -o $vpnIface -j ACCEPT")
+                pf("-A ointernal $statenew -o $vpnIface -j ACCEPT")
             );
             try {
                 my %vpnSettings = %{$rsMod->vpnSettings()};
                 push(@commands,
-                     pf("-A ointernal $new -p $vpnSettings{protocol} "
+                     pf("-A ointernal $statenew -p $vpnSettings{protocol} "
                           . "-d $vpnSettings{ipAddr} --dport $vpnSettings{port} -j ACCEPT")
             );
             } catch EBox::Exceptions::External with {
                 # Cannot contact eBox CC
                 my ($exc) = @_;
-                EBox::error("Cannot contact eBox Control Center: $exc");
+                EBox::error("Cannot contact Zentyal Cloud: $exc");
             };
             # Allow communications between ns and www
             eval "use EBox::RemoteServices::Configuration";
@@ -309,14 +328,15 @@ sub _setRemoteServices
                );
             # We are assuming just one name server
             push(@commands,
-                pf("-A ointernal $new -p udp -d $dnsServer --dport 53 -j ACCEPT")
+                pf("-A ointernal $statenew -p udp -d $dnsServer --dport 53 -j ACCEPT"),
+
             );
             # Public WWW servers to connect to
             for my $no ( 1 .. $mirrorCount ) {
                 my $site = $publicWebServer;
                 $site =~ s:\.:$no.:;
                 push(@commands,
-                    pf("-A ointernal $new -p tcp -d $site --dport 443 -j ACCEPT")
+                    pf("-A ointernal $statenew -p tcp -d $site --dport 443 -j ACCEPT")
                 );
             }
         }
@@ -458,6 +478,8 @@ sub start
 
     my @ifaces = @{$self->{net}->ifaces()};
     foreach my $ifc (@ifaces) {
+        next if ($self->{net}->ifaceMethod($ifc) eq 'bridged');
+
         if ($self->{net}->ifaceMethod($ifc) eq any('dhcp', 'ppp')) {
             push(@commands, @{$self->_setDHCP($ifc)});
             my $dnsSrvs = $self->{net}->DHCPNameservers($ifc);
@@ -480,24 +502,44 @@ sub start
         my $method = $self->{net}->ifaceMethod($if);
         $if = $self->{net}->realIface($if);
 
-        push(@commands,
-                pf("-A fnoexternal $new -i $if -j fdrop"),
-                pf("-A inoexternal $new -i $if -j idrop"),
-                pf("-A ftoexternalonly -o $if -j ACCEPT"),
+        my $input = $self->_inputIface($if);
+        my $output = $self->_outputIface($if);
+
+        unless ( $self->{net}->ifaceIsBridge($if) ) {
+            push(@commands,
+                pf("-A fnoexternal $statenew $input -j fdrop"),
+                pf("-A inoexternal $statenew $input -j idrop"),
+                pf("-A ftoexternalonly $output -j ACCEPT"),
             );
+        }
 
         next unless (_natEnabled());
 
         if ($method eq 'static') {
             my $addr = $self->{net}->ifaceAddress($if);
+            my $src = $addr;
+
+            # If it's a bridge SNAT traffic out of the network
+            if ( $self->{net}->ifaceIsBridge($if) ) {
+                my $mask = $self->{net}->ifaceNetmask($if);
+                $src = "$addr/$mask";
+            }
             push(@commands,
-                    pf("-t nat -A POSTROUTING -s ! $addr -o $if " .
-                        "-j SNAT --to $addr")
-                );
+                pf("-t nat -A POSTROUTING -s ! $src $output " .
+                   "-j SNAT --to $addr")
+            );
         } elsif (($method eq 'dhcp') or ($method eq 'ppp')) {
-            push(@commands,
-                    pf("-t nat -A POSTROUTING -o $if -j MASQUERADE")
+            if ( $self->{net}->ifaceIsBridge($if) ) {
+                push(@commands,
+                    pf("-t nat -A POSTROUTING $output -m physdev" .
+                       " ! --physdev-is-bridged -j MASQUERADE")
                 );
+            }
+            else {
+                push(@commands,
+                    pf("-t nat -A POSTROUTING $output -j MASQUERADE")
+                );
+            }
         }
     }
 
@@ -517,7 +559,7 @@ sub start
         my $port = $rule->{port};
         my $proto = $rule->{protocol};
         push(@commands,
-                pf("-A ointernal $new  -p $proto --dport $port -j ACCEPT")
+                pf("-A ointernal $statenew -p $proto --dport $port -j ACCEPT")
             );
     }
 
@@ -531,6 +573,31 @@ sub start
 
     push(@commands, @{_startIPForward()});
 
+    my @modRules = @{$self->moduleRules()};
+
+    my $model = $self->{firewall}->{'EBoxServicesRuleTable'};
+    my %enabledRules =
+        map { $model->row($_)->valueByName('rule') => 1 } @{$model->enabledRows()};
+
+    my @sortedRules = sort { $a->{'priority'} <=> $b->{'priority'} } @modRules;
+    push(@commands, map { my $r = $_->{'rule'};
+                          pf($r) if $enabledRules{$r} } @sortedRules);
+
+    root(@commands);
+}
+
+# Method: moduleRules
+#
+#       Get the rules added by the Zentyal modules through FirewallObserver
+#
+# Returns:
+#
+#      Reference to array of hashrefs { module, priority, rule }
+#
+sub moduleRules
+{
+    my ($self) = @_;
+
     my $global = EBox::Global->getInstance();
     my @modNames = @{$global->modNames};
     my @mods = @{$global->modInstancesOfType('EBox::FirewallObserver')};
@@ -538,40 +605,27 @@ sub start
     foreach my $mod (@mods) {
         my $helper = $mod->firewallHelper();
         ($helper) or next;
-        # push chain creation directly to commands so they take precedence
-        push(@commands, map { pf("-N $_") } @{$helper->chains()});
         push(@modRules,
-                @{$self->_doRuleset('nat', 'premodules', $helper->prerouting())}
+                @{$self->_doRuleset($mod, 'nat', 'premodules', $helper->prerouting())}
             );
         push(@modRules,
-                @{$self->_doRuleset('nat', 'postmodules', $helper->postrouting())}
+                @{$self->_doRuleset($mod, 'nat', 'postmodules', $helper->postrouting())}
             );
         push(@modRules,
-                @{$self->_doRuleset('filter', 'fmodules', $helper->forward())}
+                @{$self->_doRuleset($mod, 'filter', 'fmodules', $helper->forward())}
             );
         push(@modRules,
-                @{$self->_doRuleset('filter', 'iexternalmodules', $helper->externalInput())}
+                @{$self->_doRuleset($mod, 'filter', 'iexternalmodules', $helper->externalInput())}
             );
         push(@modRules,
-                @{$self->_doRuleset('filter', 'imodules', $helper->input())}
+                @{$self->_doRuleset($mod, 'filter', 'imodules', $helper->input())}
             );
         push(@modRules,
-                @{$self->_doRuleset('filter', 'omodules', $helper->output())}
-            );
-    }
-    my @sortedRules = sort { $a->{'priority'} <=> $b->{'priority'} } @modRules;
-    push(@commands, map { pf($_->{'rule'}) } @sortedRules);
-
-    # Special rule for PPPoE interfaces to avoid problems with large packets
-    foreach my $if (@{$self->{net}->pppIfaces()}) {
-        $if = $self->{net}->realIface($if);
-        push(@commands,
-                pf("-t mangle -A FORWARD -o $if -p tcp -m tcp " .
-                   "--tcp-flags SYN,RST SYN -j TCPMSS --clamp-mss-to-pmtu")
+                @{$self->_doRuleset($mod, 'filter', 'omodules', $helper->output())}
             );
     }
 
-    root(@commands);
+    return \@modRules;
 }
 
 sub _loadIptModules
@@ -583,9 +637,9 @@ sub _loadIptModules
     return \@commands;
 }
 
-sub _doRuleset # (table, chain, \@rules)
+sub _doRuleset # (module, table, chain, \@rules)
 {
-    my ($self, $table, $chain, $rules) = @_;
+    my ($self, $module, $table, $chain, $rules) = @_;
 
     my @commands;
     foreach my $r (@{$rules}) {
@@ -604,7 +658,7 @@ sub _doRuleset # (table, chain, \@rules)
             $pfrule = $r;
         }
         $pfrule = "-t $table -A $pfchain $pfrule";
-        my $r = { 'priority' => $priority, 'rule' => $pfrule };
+        my $r = { 'module' => $module, 'priority' => $priority, 'rule' => $pfrule };
         push(@commands, $r);
     }
     return \@commands;
@@ -623,9 +677,19 @@ sub _iexternalCheckInit
     my @internalIfaces = @{$self->{net}->InternalIfaces()};
     foreach my $if (@internalIfaces) {
         $if = $self->{net}->realIface($if);
+        my $input = $self->_inputIface($if);
+
         push(@commands,
-            pf("-A iexternalmodules -i $if -j RETURN"),
-            pf("-A iexternal -i $if -j RETURN"),
+            pf("-A iexternalmodules $input -j RETURN"),
+            pf("-A iexternal $input -j RETURN"),
+        );
+    }
+    foreach my $if (@{_vpnIfaces()}) {
+        my $input = $self->_inputIface($if);
+
+        push(@commands,
+            pf("-A iexternalmodules $input -j RETURN"),
+            pf("-A iexternal $input -j RETURN"),
         );
     }
     return \@commands;
@@ -652,7 +716,7 @@ sub _iexternal
 # Method: _iglobal
 #
 #	Add rules to iglobal, that is the chain to control access
-#	from the internal networks to eBox
+#	from the internal networks to Zentyal
 sub _iglobal
 {
     my ($self) = @_;
@@ -669,7 +733,7 @@ sub _iglobal
 # Method: _oglobal
 #
 #   Add rules to iglobal, that is the chain to control access
-#   from eBox to external services
+#   from Zentyal to external services
 sub _oglobal
 {
     my ($self) = @_;
@@ -714,7 +778,9 @@ sub _ffwdrules
     my @internalIfaces = @{$self->{net}->InternalIfaces()};
     foreach my $if (@internalIfaces) {
         $if = $self->{net}->realIface($if);
-        push(@commands, pf("-A ffwdrules -i $if -j RETURN"));
+        my $input = $self->_inputIface($if);
+
+        push(@commands, pf("-A ffwdrules $input -j RETURN"));
     }
     my $iptHelper = new EBox::Firewall::IptablesHelper;
     for my $rule (@{$iptHelper->ExternalToInternalRuleTable()}) {
@@ -817,6 +883,49 @@ sub _log
     return \@commands;
 }
 
+# Method: _outputIface
+#
+#   Returns iptables rule part for output interface selection
+#   Takes into account if the iface is part of a bridge
+#
+# Parameters:
+#
+#   Iface - Iface name
+#
+sub _outputIface # (iface)
+{
+    my ($self, $iface) = @_;
+
+    if ( $self->{net}->ifaceExists($iface) and
+         $self->{net}->ifaceMethod($iface) eq 'bridged' ) {
+        return  "-m physdev --physdev-out $iface";
+    } else {
+        return "-o $iface";
+    }
+}
+
+# Method: _inputIface
+#
+#   Returns iptables rule part for input interface selection
+#   Takes into account if the iface is part of a bridge
+#
+# Parameters:
+#
+#   Iface - Iface name
+#
+sub _inputIface # (iface)
+{
+    my ($self, $iface) = @_;
+
+    if ( $self->{net}->ifaceExists($iface) and
+         $self->{net}->ifaceMethod($iface) eq 'bridged' ) {
+        return  "-m physdev --physdev-in $iface";
+    } else {
+        return "-i $iface";
+    }
+}
+
+
 # Method: _natEnabled
 #
 #   Fetch value to enable NAT
@@ -834,4 +943,19 @@ sub _natEnabled
     }
 }
 
+# Method: _vpnIfaces
+#
+#   Fetch vpn interfaces
+sub _vpnIfaces
+{
+    my $gl = EBox::Global->getInstance();
+    if ($gl->modExists('openvpn')) {
+        my $vpn = $gl->modInstance('openvpn');
+        $vpn->initializeInterfaces();
+        return [map {$_->iface() } $vpn->activeDaemons()];
+    } else {
+        return [];
+    }
+}
+
 1;
